package net.wanji.web.service.impl;

import net.wanji.web.common.util.date.TimeArrayUtil;
import net.wanji.web.mapper.FlowQueueMapper;
import net.wanji.web.po.FlowQueuePO;
import net.wanji.web.service.SignalEvaluateService;
import net.wanji.web.vo.signalEvaluate.FlowQueueOutVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2022/11/3 16:15
 */
@Service
public class SignalEvaluateServiceImpl implements SignalEvaluateService {

    @Autowired
    FlowQueueMapper flowQueueMapper;

    @Override
    public FlowQueueOutVO flowQueue(String adCode, String crossId, Date currentTime, Integer timeType, Integer dataType) {
        FlowQueueOutVO flowQueueOutVO = new FlowQueueOutVO();
        if (timeType == 1) { // 按日
            if (dataType == 1) { // 流量
                flowQueueOutVO = getDayFlow(adCode, crossId, currentTime);
            } else if (dataType == 2) { // 排队
                flowQueueOutVO = getDayQueue(adCode, crossId, currentTime);
            }
        } else if (timeType == 2) { // todo 按周

        } else if (timeType == 3) { // todo 按月

        }
        return flowQueueOutVO;
    }

    private FlowQueueOutVO getDayQueue(String adCode, String crossId, Date currentTime) {
        List<String> time = TimeArrayUtil.getTimeArray();
        double[] toDay = new double[24];
        double[] yesterday = new double[24];
        double[] lastWeekDay = new double[24];


        List<FlowQueuePO> dayQueueTodayPOList = flowQueueMapper.getDayQueueToday(adCode, crossId, currentTime);
        fillArray(time, toDay, dayQueueTodayPOList);

        List<FlowQueuePO> dayQueueYesterdayPOList = flowQueueMapper.getDayQueueYesterday(adCode, crossId, currentTime);
        fillArray(time, yesterday, dayQueueYesterdayPOList);

        List<FlowQueuePO> dayQueueLastWeekDayPOList = flowQueueMapper.getDayQueueLastWeekDay(adCode, crossId, currentTime);
        fillArray(time, lastWeekDay, dayQueueLastWeekDayPOList);

        FlowQueueOutVO flowQueueOutVO = new FlowQueueOutVO();
        flowQueueOutVO.setTime(time);
        flowQueueOutVO.setToDay(toDay);
        flowQueueOutVO.setYesterday(yesterday);
        flowQueueOutVO.setLastWeekDay(lastWeekDay);
        return flowQueueOutVO;
    }

    private FlowQueueOutVO getDayFlow(String adCode, String crossId, Date currentTime) {
        List<String> time = TimeArrayUtil.getTimeArray();
        double[] toDay = new double[24];
        double[] yesterday = new double[24];
        double[] lastWeekDay = new double[24];


        List<FlowQueuePO> dayFlowTodayPOList = flowQueueMapper.getDayFlowToday(adCode, crossId, currentTime);
        fillArray(time, toDay, dayFlowTodayPOList);

        List<FlowQueuePO> dayFlowYesterdayPOList = flowQueueMapper.getDayFlowYesterday(adCode, crossId, currentTime);
        fillArray(time, yesterday, dayFlowYesterdayPOList);

        List<FlowQueuePO> dayFlowLastWeekDayPOList = flowQueueMapper.getDayFlowLastWeekDay(adCode, crossId, currentTime);
        fillArray(time, lastWeekDay, dayFlowLastWeekDayPOList);

        FlowQueueOutVO flowQueueOutVO = new FlowQueueOutVO();
        flowQueueOutVO.setTime(time);
        flowQueueOutVO.setToDay(toDay);
        flowQueueOutVO.setYesterday(yesterday);
        flowQueueOutVO.setLastWeekDay(lastWeekDay);
        return flowQueueOutVO;
    }

    private void fillArray(List<String> time, double[] arr, List<FlowQueuePO> list) {
        changeTimeStr(list);
        // 填入结果数组
        for (FlowQueuePO flowQueuePO : list) {
            int i = time.indexOf(flowQueuePO.getTime());
            arr[i] = flowQueuePO.getCount();
        }
    }

    private void changeTimeStr(List<FlowQueuePO> dayFlowTodayPOList) {
        for (FlowQueuePO flowQueuePO : dayFlowTodayPOList) {
            // 替换时间字符串
            String controlTime = flowQueuePO.getTime();
            if (':' == controlTime.charAt(1)) {
                controlTime = "0" + controlTime;
            }
            flowQueuePO.setTime(controlTime);
        }
    }
}
