package net.wanji.web.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import net.wanji.web.common.enums.*;
import net.wanji.web.common.util.CrossUtil;
import net.wanji.web.entity.TBaseCrossInfo;
import net.wanji.web.mapper.*;
import net.wanji.web.po.*;
import net.wanji.web.service.SituationDetectionService;
import net.wanji.web.vo.situationDetection.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 态势监测服务类
 *
 * @author Kent HAN
 * @date 2022/10/26 13:48
 */
@Service
public class SituationDetectionServiceImpl implements SituationDetectionService {

    @Autowired
    OptRecordsMapper optRecordsMapper;
    @Autowired
    CrossAlarmMapper crossAlarmMapper;
    @Autowired
    EventAlarmMapper eventAlarmMapper;
    @Autowired
    ControlHistMapper controlHistMapper;
    @Autowired
    VehicleTraceMapper vehicleTraceMapper;
    @Autowired
    VideoDeviceInfoMapper videoDeviceInfoMapper;
    @Autowired
    GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    @Autowired
    AllDeviceStatusMapper allDeviceStatusMapper;
    @Autowired
    GreenwaveCrossMapper greenwaveCrossMapper;

    @Autowired
    TBaseCrossInfoMapper tBaseCrossInfoMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public OptRecordsOutVO optRecords(String adCode, Date adaptTime) {
        OptRecordsOutVO optRecordsOutVO = new OptRecordsOutVO();
        List<OptRecordsOutVO.TrafficIndexVO> trafficIndexVOList = new ArrayList<>();

        List<BaseAreaDataPO> baseAreaDataPOList = optRecordsMapper.selectByAdCodeAndTime(adCode, adaptTime);
        double optNumSum = 0.0;
        double speedSum = 0.0;
        int speedCount = 0;
        for (BaseAreaDataPO baseAreaDataPO : baseAreaDataPOList) {
            // 加总优化次数
            Double optNum = baseAreaDataPO.getOptNum();
            optNumSum += optNum;

            Double speed = baseAreaDataPO.getSpeed();
            speedSum += speed;
            speedCount += 1;

            // 构造交通指数列表
            OptRecordsOutVO.TrafficIndexVO trafficIndexVO = new OptRecordsOutVO.TrafficIndexVO();
            Date adaptFullTime = baseAreaDataPO.getAdaptTime();
            String adaptFullTimeStr = sdf.format(adaptFullTime);
            // 返回小时分钟
            String timeStr = adaptFullTimeStr.substring(11, 16);
            trafficIndexVO.setTime(timeStr);
            Double trafficIndex = baseAreaDataPO.getTrafficIndex();
            trafficIndexVO.setIndex(trafficIndex);
            trafficIndexVOList.add(trafficIndexVO);
        }
        // 计算平均速度
        double averageSpeed = speedSum / speedCount;

        optRecordsOutVO.setOptTimes((int) optNumSum);
        optRecordsOutVO.setSpeed((int) averageSpeed);
        optRecordsOutVO.setTrafficIndexVOList(trafficIndexVOList);

        return optRecordsOutVO;
    }

    @Override
    public List<CrossAlarmOutVO> crossAlarm() {
        List<CrossAlarmPO> crossAlarmPOList = crossAlarmMapper.selectStatusNotZero();
        List<CrossAlarmOutVO> crossAlarmOutVOList = new ArrayList<>();
        for (CrossAlarmPO crossAlarmPO : crossAlarmPOList) {
            // 获取状态
            Integer isUnbalance = crossAlarmPO.getIsUnbalance();
            Integer isSpillover = crossAlarmPO.getIsSpillover();
            Integer isCongestion = crossAlarmPO.getIsCongestion();

            if (isUnbalance == 1) {
                String msg = CrossAlarmEnum.ONE.getMsg();
                Integer minuteDuration = crossAlarmPO.getUnbalanceTime();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
            if (isCongestion == 1) {
                String msg = CrossAlarmEnum.TWO.getMsg();
                Integer minuteDuration = crossAlarmPO.getCongestionTime();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
            if (isSpillover == 1) {
                String msg = CrossAlarmEnum.THREE.getMsg();
                Integer minuteDuration = crossAlarmPO.getSpilloverTime();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
            if ("4".equals(crossAlarmPO.getStatus())) {
                String msg = CrossAlarmEnum.FOUR.getMsg();
                Integer minuteDuration = crossAlarmPO.getDuration();
                addCrossAlarmVO(crossAlarmOutVOList, crossAlarmPO, msg, minuteDuration);
            }
        }
        CrossAlarmComparator crossAlarmComparator = new CrossAlarmComparator();
        crossAlarmOutVOList.sort(crossAlarmComparator);

        return crossAlarmOutVOList;
    }

    private void addCrossAlarmVO(List<CrossAlarmOutVO> crossAlarmOutVOList, CrossAlarmPO crossAlarmPO, String msg,
                                 Integer minuteDuration) {
        CrossAlarmOutVO crossAlarmOutVO = new CrossAlarmOutVO();
        // 根据路口ID获取路口名称
        String crossId = crossAlarmPO.getCrossId();
        TBaseCrossInfo tBaseCrossInfo = tBaseCrossInfoMapper.selectByPrimaryKey(crossId);
        crossAlarmOutVO.setCrossName(tBaseCrossInfo.getName());
        // 从例如POINT(116.29959807 39.95929275)中获取经纬度
        String location = tBaseCrossInfo.getLocation();
        double[] lonLat = CrossUtil.getLonLat(location);
        crossAlarmOutVO.setLongitude(lonLat[0]);
        crossAlarmOutVO.setLatitude(lonLat[1]);

        crossAlarmOutVO.setCrossStatus(msg);
        // 发生时间
        Date date = DateUtil.date();
        Date startTime = DateUtil.offset(date, DateField.MINUTE, -minuteDuration);
        String timeStr = sdf.format(startTime);
        String substring = timeStr.substring(11, 16); // 截取小时分钟
        crossAlarmOutVO.setStartTime(substring);

        crossAlarmOutVO.setDuration(minuteDuration);
        crossAlarmOutVOList.add(crossAlarmOutVO);
    }

    @Override
    public List<EventAlarmOutVO> eventAlarm(Date todayTime) throws ParseException {
        List<EventAlarmPO> eventAlarmPOList = eventAlarmMapper.selectByTodayTime(todayTime);
        List<EventAlarmOutVO> eventAlarmOutVOList = new ArrayList<>();
        for (EventAlarmPO eventAlarmPO : eventAlarmPOList) {
            EventAlarmOutVO eventAlarmOutVO = new EventAlarmOutVO();
            eventAlarmOutVO.setEventDesc(eventAlarmPO.getEventDesc());
            // 事件类型
            Integer typeCode = eventAlarmPO.getType();
            String msgByCode = EventAlarmEnum.getMsgByCode(typeCode);
            eventAlarmOutVO.setType(msgByCode);

            Date endTime = eventAlarmPO.getEndTime();
            if (typeCode == 1 || typeCode == 2) {
                // 发生时间
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String substring = startTimeStr.substring(11, 16);
                eventAlarmOutVO.setStartTime(substring);
                // 时长
                if (endTime == null) {
                    Date nowTime = DateUtil.date();

                    // todo 时间为演示用，待删除
                    Date parse = sdf.parse("2022-10-27 09:56:23");
                    nowTime = parse;

                    long between = DateUtil.between(startTime, nowTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                } else {
                    long between = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                }
            }
            if (typeCode == 3 || typeCode == 4) {
                // 时间范围
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String startTimeSubStr = startTimeStr.substring(0, 16); // 截取到分钟
                String endTimeStr = sdf.format(endTime);
                String endTimeSubStr = endTimeStr.substring(0, 16); // 截取到分钟
                eventAlarmOutVO.setTimeSpan(startTimeSubStr + "~" + endTimeSubStr);
            }

            Integer sourceCode = eventAlarmPO.getSource();
            String sourceStr = EventAlarmSourceEnum.getMsgByCode(sourceCode);
            eventAlarmOutVO.setSource(sourceStr);
            eventAlarmOutVO.setCoordinate(eventAlarmPO.getWkt());

            eventAlarmOutVOList.add(eventAlarmOutVO);

        }
        return eventAlarmOutVOList;
    }

    @Override
    public List<ControlHistOutVO> controlHist(String crossId, String currentTime) throws ParseException {
        List<ControlHistOutVO> controlHistOutVOList = new ArrayList<>();
        Date todayDate = sdf.parse(currentTime);
        List<ControlHistPO> controlHistPOList = controlHistMapper.selectByCrossId(crossId, todayDate);
        // 替换时间字符串
        for (ControlHistPO controlHistPO : controlHistPOList) {
            String controlTime = controlHistPO.getControlTime();
            if (controlTime.endsWith(":0")) {
                controlTime += "0";
            }
            if (':' == controlTime.charAt(1)) {
                controlTime = "0" + controlTime;
            }
            controlHistPO.setControlTime(controlTime);
        }
        // 按时间聚合
        String tmpTime = "";
        Map<String, Integer> typeStat = new LinkedHashMap<>();
        // 初始化Map
        initTypeStat(typeStat);
        for (int i = 0; i < controlHistPOList.size(); i++) {
            ControlHistPO controlHistPO = controlHistPOList.get(i);
            String controlTime = controlHistPO.getControlTime();
            if (tmpTime.equals(controlTime)) {
                // 构造Map
                Integer controlType = controlHistPO.getControlType();
                String typeStr = ControlHistEnum.getMsgByCode(controlType);
                Integer typeCount = controlHistPO.getTypeCount();
                typeStat.put(typeStr, typeCount);
            } else if ("".equals(tmpTime)) {
                initTypeStat(typeStat);
                tmpTime = controlTime;
                i--;
            } else {
                // 构造一条记录
                Map<String, Integer> resTypeStat = new LinkedHashMap<>(typeStat);
                addControlHist(controlHistOutVOList, tmpTime, resTypeStat);

                initTypeStat(typeStat);
                tmpTime = controlTime;
                i--;
            }
        }
        // 构造最后一条记录
        addControlHist(controlHistOutVOList, tmpTime, typeStat);
        controlHistOutVOList.sort(Comparator.comparing(
                ControlHistOutVO::getTime, Comparator.nullsLast(Comparator.naturalOrder())));

        return controlHistOutVOList;
    }

    private void initTypeStat(Map<String, Integer> typeStat) {
        typeStat.clear();
        for (ControlHistEnum value : ControlHistEnum.values()) {
            typeStat.put(value.getMsg(), 0);
        }
    }

    private void addControlHist(List<ControlHistOutVO> controlHistOutVOList,
                                String tmpTime, Map<String, Integer> typeStat) {
        ControlHistOutVO controlHistOutVO = new ControlHistOutVO();
        controlHistOutVO.setTime(tmpTime);
        controlHistOutVO.setTypeStat(typeStat);
        controlHistOutVOList.add(controlHistOutVO);
    }

    @Override
    public List<VehicleTraceOutVO> vehicleTrace(String adCode, String currentTime) throws ParseException {
        List<VehicleTraceOutVO> vehicleTraceOutVOList = new ArrayList<>();
        Date todayDate = sdf.parse(currentTime);
        List<VehicleTracePO> vehicleTracePOList = vehicleTraceMapper.selectByAdCode(adCode, todayDate);
        // 替换时间字符串
        for (VehicleTracePO vehicleTracePO : vehicleTracePOList) {
            String traceTime = vehicleTracePO.getTraceTime();
            if (traceTime.endsWith(":0")) {
                traceTime += "0";
            }
            if (':' == traceTime.charAt(1)) {
                traceTime = "0" + traceTime;
            }
            vehicleTracePO.setTraceTime(traceTime);
        }
        for (VehicleTracePO vehicleTracePO : vehicleTracePOList) {
            VehicleTraceOutVO vehicleTraceOutVO = new VehicleTraceOutVO();
            vehicleTraceOutVO.setTraceTime(vehicleTracePO.getTraceTime());
            vehicleTraceOutVO.setVehicleCount(vehicleTracePO.getVehicleCount());
            vehicleTraceOutVOList.add(vehicleTraceOutVO);
        }
        vehicleTraceOutVOList.sort(Comparator.comparing(
                VehicleTraceOutVO::getTraceTime, Comparator.nullsLast(Comparator.naturalOrder())));

        return vehicleTraceOutVOList;
    }

    @Override
    public List<VideoDeviceInfoOutVO> videoDeviceInfo(String adCode) {
        List<VideoDeviceInfoPO> videoDeviceInfoPOList = videoDeviceInfoMapper.selectVideoDevices(adCode);
        List<VideoDeviceInfoOutVO> videoDeviceInfoOutVOList = new ArrayList<>();
        for (VideoDeviceInfoPO videoDeviceInfoPO : videoDeviceInfoPOList) {
            VideoDeviceInfoOutVO videoDeviceInfoOutVO = new VideoDeviceInfoOutVO();
            videoDeviceInfoOutVO.setCode(videoDeviceInfoPO.getCode());
            videoDeviceInfoOutVO.setName(videoDeviceInfoPO.getName());
            Integer status = videoDeviceInfoPO.getStatus();
            videoDeviceInfoOutVO.setStatus(DeviceStatusEnum.getMsgByCode(status));
            videoDeviceInfoOutVO.setLocation(videoDeviceInfoPO.getLocation());
            videoDeviceInfoOutVOList.add(videoDeviceInfoOutVO);
        }
        return videoDeviceInfoOutVOList;
    }

    @Override
    public List<GreenwaveRealtimeOutVO> greenwaveRealtime(String adCode, String currentTime) throws ParseException {
        Date todayDate = sdf.parse(currentTime);
        List<GreenwaveRealtimePO> greenwaveRealtimePOList = greenwaveRealtimeMapper.selectByAdCodeAndCurrentTime(adCode, todayDate);
        List<GreenwaveRealtimeOutVO> greenwaveRealtimeOutVOList = new ArrayList<>();
        for (GreenwaveRealtimePO greenwaveRealtimePO : greenwaveRealtimePOList) {
            GreenwaveRealtimeOutVO greenwaveRealtimeOutVO = new GreenwaveRealtimeOutVO();
            greenwaveRealtimeOutVO.setId(greenwaveRealtimePO.getGreenwaveId());
            BeanUtils.copyProperties(greenwaveRealtimePO, greenwaveRealtimeOutVO);
            greenwaveRealtimeOutVOList.add(greenwaveRealtimeOutVO);
        }

        return greenwaveRealtimeOutVOList;
    }

    @Override
    public Map<String, Map<String, Integer>> allDeviceStatus(String adCode) {
        List<AllDeviceStatusPO> allDeviceStatusPOList = allDeviceStatusMapper.selectAllDeviceStatus(adCode);
        Map<String, Map<String, Integer>> res = new LinkedHashMap<>();
        // 初始化Map
        for (DeviceTypeEnum value : DeviceTypeEnum.values()) {
            res.put(value.getMsg(), null);
        }
        Map<Integer, List<AllDeviceStatusPO>> collect = allDeviceStatusPOList.stream()
                .collect(Collectors.groupingBy(AllDeviceStatusPO::getType));
        for (Map.Entry<Integer, List<AllDeviceStatusPO>> entry : collect.entrySet()) {
            Map<String, Integer> valueMap = new LinkedHashMap<>();
            // 初始化Map
            for (DeviceStatusEnum value : DeviceStatusEnum.values()) {
                valueMap.put(value.getMsg(), 0);
            }
            for (AllDeviceStatusPO allDeviceStatusPO : entry.getValue()) {
                Integer status = allDeviceStatusPO.getStatus();
                Integer statusCount = allDeviceStatusPO.getStatusCount();
                valueMap.put(DeviceStatusEnum.getMsgByCode(status), statusCount);
            }
            Integer key = entry.getKey();
            res.put(DeviceTypeEnum.getMsgByCode(key), valueMap);
        }

        return res;
    }

    @Override
    public List<GreenwaveCrossOutVO> greenwaveCross(String adCode, String currentTime, Integer id) {
        List<GreenwaveCrossOutVO> greenwaveCrossOutVOList = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(adCode, currentTime, id);
        for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOList) {
            GreenwaveCrossOutVO greenwaveCrossOutVO = new GreenwaveCrossOutVO();
            greenwaveCrossOutVO.setName(greenwaveCrossPO.getName());
            // 构造坐标
            String location = greenwaveCrossPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            greenwaveCrossOutVO.setLongitude(lonLat[0]);
            greenwaveCrossOutVO.setLatitude(lonLat[1]);

            greenwaveCrossOutVOList.add(greenwaveCrossOutVO);
        }

        return greenwaveCrossOutVOList;
    }

    @Override
    public List<CrossDeviceStatusInfoOutVO> crossDeviceStatusInfo() {
        List<CrossDeviceStatusInfoOutVO> results = tBaseCrossInfoMapper.listCrossDeviceStatusInfo();
        return results;
    }

    @Override
    public TBaseCrossInfo selectCrossInfoById(String signalId) {
        return tBaseCrossInfoMapper.selectById(signalId);
    }
}

class CrossAlarmComparator implements Comparator<CrossAlarmOutVO> {

    @Override
    public int compare(CrossAlarmOutVO o1, CrossAlarmOutVO o2) {
        // 先按路口名称排序，再按溢出-拥堵-失衡排序
        int flag = o1.getCrossName().compareTo(o2.getCrossName());
        if (flag == 0) {
            String crossStatus1 = o1.getCrossStatus();
            Integer codeInt1 = CrossAlarmEnum.getCodeIntByMsg(crossStatus1);
            String crossStatus2 = o2.getCrossStatus();
            Integer codeInt2 = CrossAlarmEnum.getCodeIntByMsg(crossStatus2);
            return codeInt2 - codeInt1;
        }
        return flag;
    }


}
